// Copyright 2024 The go-ethereum Authors
// This file is part of the go-ethereum library.
//
// The go-ethereum library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// The go-ethereum library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with the go-ethereum library. If not, see <http://www.gnu.org/licenses/>.

package live

import (
	"code.wuban.net.cn/cmpchain/ethtracer/evm/evmtypes"
	"code.wuban.net.cn/cmpchain/ethtracer/params"
	"code.wuban.net.cn/cmpchain/ethtracer/tracers"
	"code.wuban.net.cn/cmpchain/ethtracer/tracing"
	"encoding/json"
	metatypes "github.com/CaduceusMetaverseProtocol/MetaTypes/types"
	"github.com/CaduceusMetaverseProtocol/MetaTypes/types/coretype"
	"math/big"
)

func init() {
	tracers.LiveDirectory.Register("noop", newNoopTracer)
}

// noop is a no-op live tracer. It's there to
// catch changes in the tracing interface, as well as
// for testing live tracing performance. Can be removed
// as soon as we have a real live tracer.
type noop struct{}

func newNoopTracer(_ json.RawMessage) (*tracing.Hooks, error) {
	t := &noop{}
	return &tracing.Hooks{
		OnTxStart:        t.OnTxStart,
		OnTxEnd:          t.OnTxEnd,
		OnEnter:          t.OnEnter,
		OnExit:           t.OnExit,
		OnOpcode:         t.OnOpcode,
		OnFault:          t.OnFault,
		OnGasChange:      t.OnGasChange,
		OnBlockchainInit: t.OnBlockchainInit,
		OnBlockStart:     t.OnBlockStart,
		OnBlockEnd:       t.OnBlockEnd,
		OnSkippedBlock:   t.OnSkippedBlock,
		OnGenesisBlock:   t.OnGenesisBlock,
		OnBalanceChange:  t.OnBalanceChange,
		OnNonceChange:    t.OnNonceChange,
		OnCodeChange:     t.OnCodeChange,
		OnStorageChange:  t.OnStorageChange,
		OnLog:            t.OnLog,
		OnBlockHashRead:  t.OnBlockHashRead,
	}, nil
}

func (t *noop) OnOpcode(pc uint64, op byte, gas, cost uint64, scope tracing.OpContext, rData []byte, depth int, err error) {
}

func (t *noop) OnFault(pc uint64, op byte, gas, cost uint64, _ tracing.OpContext, depth int, err error) {
}

func (t *noop) OnEnter(depth int, typ byte, from metatypes.Address, to metatypes.Address, input []byte, gas uint64, value *big.Int) {
}

func (t *noop) OnExit(depth int, output []byte, gasUsed uint64, err error, reverted bool) {
}

func (t *noop) OnTxStart(vm *tracing.VMContext, tx *evmtypes.Transaction, from metatypes.Address) {
}

func (t *noop) OnTxEnd(receipt *evmtypes.Receipt, err error) {
}

func (t *noop) OnBlockStart(ev tracing.BlockEvent) {
}

func (t *noop) OnBlockEnd(err error) {
}

func (t *noop) OnSkippedBlock(ev tracing.BlockEvent) {}

func (t *noop) OnBlockchainInit(chainConfig *params.ChainConfig) {
}

func (t *noop) OnGenesisBlock(b *evmtypes.Block, alloc coretype.GenesisAlloc) {
}
func (t *noop) OnBalanceChange(a metatypes.Address, prev, new *big.Int, reason tracing.BalanceChangeReason) {
}

func (t *noop) OnNonceChange(a metatypes.Address, prev, new uint64) {
}

func (t *noop) OnCodeChange(a metatypes.Address, prevCodeHash metatypes.Hash, prev []byte, codeHash metatypes.Hash, code []byte) {
}

func (t *noop) OnStorageChange(a metatypes.Address, k, prev, new metatypes.Hash) {
}

func (t *noop) OnLog(l *evmtypes.Log) {

}

func (t *noop) OnBlockHashRead(number uint64, hash metatypes.Hash) {}

func (t *noop) OnGasChange(old, new uint64, reason tracing.GasChangeReason) {
}
